package net.es.oscars.policy.rmi;

import java.io.PrintStream;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Properties;

import net.es.oscars.PropHandler;
import net.es.oscars.policy.client.CheckPolicyRequest;
import net.es.oscars.policy.client.CheckPolicyResponse;
import net.es.oscars.policy.client.DOMRequester;
import net.es.oscars.rmi.BaseRmiClient;

import org.apache.log4j.Logger;
import org.w3c.dom.Element;

/**
 * Client to policy service RMI server
 * 
 * @author Andrew Lake (alake@internet2.edu)
 */
public class PolicyRmiClient extends BaseRmiClient implements PolicyRmiInterface{
    private Logger log = Logger.getLogger(PolicyRmiInterface.class);

    public PolicyRmiClient(){
        this.log = Logger.getLogger(this.getClass());
    }

    /**
     * Creates the remote object
     * 
     * @throws RemoteException
     */
    public void init() throws RemoteException{
        this.log.debug("Starting PolicyRmi connection");

        PropHandler propHandler = new PropHandler("oscars.properties");
        Properties props = propHandler.getPropertyGroup("rmi.policy", true);
        this.setProps(props);
        this.rmiServerName = PolicyRmiInterface.registeredServerName;
        this.serviceName="Policy RMI Client";

        super.configure();
        Remote remote = super.startConnection();
        if (this.connected) {
            this.setRemote((PolicyRmiInterface) remote);
            super.setRemote(remote);
        }
        this.log.debug("PolicyRmiClient.init().end");
    }

    /**
     * Contacts RMI server to determine whether action is allowed for requester(s)
     * 
     * @param request the check policy request with DOM elements
     * @return the result of the policy check
     * @throws RemoteException
     */
    public CheckPolicyResponse checkPolicy(CheckPolicyRequest<DOMRequester,Element[]> request) throws RemoteException {
        this.log.info("checkPolicy.start");
        String methodName = "CheckPolicy";
        CheckPolicyResponse response =  null;
        this.verifyRmiConnection(methodName);
        try {
            response = ((PolicyRmiInterface)this.remote).checkPolicy(request);
        } catch (Exception e) {
            throw  new RemoteException(e.toString());
        }
        this.log.info("checkPolicy.end");

        return response;
    }

    public void loadPolicy(String filename, Boolean verbose)
            throws RemoteException {
        this.log.info("loadPolicy.start");
        String methodName = "loadPolicy";
        this.verifyRmiConnection(methodName);
        try {
            ((PolicyRmiInterface)this.remote).loadPolicy(filename, verbose);
        } catch (Exception e) {
            this.log.error(e.getMessage());
            throw  new RemoteException(e.getMessage());
        }
        this.log.info("loadPolicy.end");
        
    }
    
}
